/**********************************************************************
 * 
 * ttest.c
 *
 * Copyright (c) Numerical Recipe
 *
 *
 * C functions for the student t-test
 *
 **********************************************************************/

#include "ttest.h"
#include <R_ext/Error.h>

static double dsqrarg;
#define DSQR(a) ((dsqrarg=(a)) == 0.0 ? 0.0 : dsqrarg*dsqrarg)


/**********************************************************************
 * 
 * betacf
 *
 * continued fraction used by betai
 * 
 **********************************************************************/

#define MAXIT 100
#define EPS 3.0e-7
#define FPMIN 1.0e-30

double betacf(double a, double b, double x)
{
	int m,m2;
	double aa,c,d,del,h,qab,qam,qap;

	qab=a+b;
	qap=a+1.0;
	qam=a-1.0;
	c=1.0;
	d=1.0-qab*x/qap;
	if (fabs(d) < FPMIN) d=FPMIN;
	d=1.0/d;
	h=d;
	for (m=1;m<=MAXIT;m++) {
		m2=2*m;
		aa=m*(b-m)*x/((qam+m2)*(a+m2));
		d=1.0+aa*d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=1.0+aa/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		h *= d*c;
		aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
		d=1.0+aa*d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=1.0+aa/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=d*c;
		h *= del;
		if (fabs(del-1.0) < EPS) break;
	}
	if (m > MAXIT) error("a or b too big, or MAXIT too small in betacf");
	return h;
}
#undef MAXIT
#undef EPS
#undef FPMIN

/**********************************************************************
 * 
 * gammln
 *
 * logarithm of gamma function 
 * 
 **********************************************************************/

double gammln(double xx)
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}

/**********************************************************************
 * 
 * betai
 *
 * incomplete beta function 
 * 
 **********************************************************************/

double betai(double a, double b, double x)
{
	double betacf(double a, double b, double x);
	double gammln(double xx);
	double bt;

	if (x < 0.0 || x > 1.0) error("Bad x in routine betai");
	if (x == 0.0 || x == 1.0) bt=0.0;
	else
		bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
	if (x < (a+1.0)/(a+b+2.0))
		return bt*betacf(a,b,x)/a;
	else
		return 1.0-bt*betacf(b,a,1.0-x)/b;
}

/**********************************************************************
 * 
 * avevar
 *
 * compute the mean and variance 
 * 
 **********************************************************************/
void avevar(double *data, int n, double *ave, double *var)
{
	int j;
	double s,ep;

	for (*ave=0.0,j=0;j<n;j++) *ave += data[j];
	*ave /= n;
	*var=ep=0.0;
	
	for (j=0;j<n;j++) {
		s=data[j]-(*ave);
		ep += s;
		*var += s*s;
	}
	*var=(*var-ep*ep/n)/(n-1);
}

/**********************************************************************
 * 
 * tctest
 * 
 * t-test with combined variance 
 *
 **********************************************************************/
void tctest(double *data1, int *l1, double *data2, int *l2, double *t, double *prob)
{
	void avevar(double data[], int n, double *ave, double *var);
	double betai(double a, double b, double x);
	double var1,var2,svar,df,ave1,ave2;
  int n1 = *l1;
  int n2 = *l2;
	avevar(data1,n1,&ave1,&var1);
	avevar(data2,n2,&ave2,&var2);
	df=n1+n2-2;
	svar=((n1-1)*var1+(n2-1)*var2)/df;
	*t=(ave1-ave2)/sqrt(svar*(1.0/n1+1.0/n2));
	*prob=betai(0.5*df,0.5,df/(df+(*t)*(*t)));
}

/**********************************************************************
 * 
 * ttest
 * 
 * t-test with un-equal variance 
 *
 **********************************************************************/
void tutest(double *data1, int* l1, double *data2, int* l2, double *t, double *prob)
{
	void avevar(double data[], int n, double *ave, double *var);
	double betai(double a, double b, double x);
	double var1,var2,df,ave1,ave2;
  int n1 = *l1;
  int n2 = *l2;
	avevar(data1,n1,&ave1,&var1);
	avevar(data2,n2,&ave2,&var2);
	*t=(ave1-ave2)/sqrt(var1/n1+var2/n2);
	df=DSQR(var1/n1+var2/n2)/(DSQR(var1/n1)/(n1-1)+DSQR(var2/n2)/(n2-1));
	*prob=betai(0.5*df,0.5,df/(df+DSQR(*t)));
}
